@@ -20,6 +20,7 @@ use Doctrine\ORM\Version;
 use Symfony\Bridge\Doctrine\DataCollector\DoctrineDataCollector as BaseCollector;
 use Symfony\Component\HttpFoundation\Request;
 use Symfony\Component\HttpFoundation\Response;
+use Symfony\Component\VarDumper\Cloner\Data;
 
 /**
  * DoctrineDataCollector.
@@ -136,6 +137,17 @@ class DoctrineDataCollector extends BaseCollector
             }
         }
 
+        // HttpKernel < 3.2 compatibility layer
+        if (method_exists($this, 'cloneVar')) {
+            // Might be good idea to replicate this block in doctrine bridge so we can drop this from here after some time.
+            // This code is compatible with such change, because cloneVar is supposed to check if input is already cloned.
+            foreach ($this->data['queries'] as &$queries) {
+                foreach ($queries as &$query) {
+                    $query['params'] = $this->cloneVar($query['params']);
+                }
+            }
+        }
+
         $this->data['entities'] = $entities;
         $this->data['errors'] = $errors;
         $this->data['caches'] = $caches;
@@ -222,15 +234,26 @@ class DoctrineDataCollector extends BaseCollector
             });
             $groupedQueries[$connection] = $connectionGroupedQueries;
         }
+
         foreach ($groupedQueries as $connection => $queries) {
             foreach ($queries as $i => $query) {
-                $groupedQueries[$connection][$i]['executionPercent'] = $query['executionMS'] / $totalExecutionMS * 100;
+                $groupedQueries[$connection][$i]['executionPercent'] =
+                    $this->executionTimePercentage($query['executionMS'], $totalExecutionMS);
             }
         }
 
         return $groupedQueries;
     }
 
+    private function executionTimePercentage($executionTimeMS, $totalExecutionTimeMS)
+    {
+        if ($totalExecutionTimeMS === 0.0 || $totalExecutionTimeMS === 0) {
+            return 0;
+        }
+
+        return $executionTimeMS / $totalExecutionTimeMS * 100;
+    }
+
     public function getGroupedQueryCount()
     {
         $count = 0;
