@@ -47,6 +47,7 @@ class EntityListenerPass implements CompilerPassInterface
                 }
 
                 $resolver = $container->findDefinition($resolverId);
+                $resolver->setPublic(true);
 
                 if (isset($attributes['entity']) && isset($attributes['event'])) {
                     $this->attachToListener($container, $name, $id, $attributes);
@@ -55,10 +56,6 @@ class EntityListenerPass implements CompilerPassInterface
                 if (isset($attributes['lazy']) && $attributes['lazy']) {
                     $listener = $container->findDefinition($id);
 
-                    if (!$listener->isPublic()) {
-                        throw new InvalidArgumentException(sprintf('The service "%s" must be public as this entity listener is lazy-loaded.', $id));
-                    }
-
                     if ($listener->isAbstract()) {
                         throw new InvalidArgumentException(sprintf('The service "%s" must not be abstract as this entity listener is lazy-loaded.', $id));
                     }
@@ -77,6 +74,8 @@ class EntityListenerPass implements CompilerPassInterface
                         );
                     }
 
+                    $listener->setPublic(true);
+
                     $resolver->addMethodCall('registerService', array($listener->getClass(), $id));
                 } else {
                     $resolver->addMethodCall('register', array(new Reference($id)));
